@@ -2,13 +2,16 @@
 
 """ Various utilities, converters etc., to help video calibration. """
 
-# pylint:disable=invalid-name
+# pylint:disable=invalid-name,logging-not-lazy
 
 import collections
+import logging
 import numpy as np
 import cv2
 import sksurgerycore.transforms.matrix as skcm
 
+LOGGER = logging.getLogger(__name__)
+
 
 def convert_numpy2d_to_opencv(image_points):
     """
@@ -357,24 +360,29 @@ def detect_points_in_canonical_space(point_detector,
             images[j],
             camera_matrix,
             distortion_coefficients,
+            None,
             camera_matrix
         )
 
-        ids, obj_pts, img_pts = point_detector.get_points(undistorted)
+        ids, obj_pts, img_pts = \
+            point_detector.get_points(undistorted, is_distorted=False)
 
         if ids is not None and ids.shape[0] >= minimum_points_per_frame:
 
             common_points = match_points_by_id(ids, img_pts,
                                                reference_ids,
                                                reference_image_points)
+
             homography, _ = \
                 cv2.findHomography(common_points[0:, 0:2],
                                    common_points[0:, 2:4])
+
             warped = cv2.warpPerspective(undistorted,
                                          homography,
                                          reference_image_size,)
 
-            ids, obj_pts, img_pts = point_detector.get_points(warped)
+            ids, obj_pts, img_pts = \
+                point_detector.get_points(warped, is_distorted=False)
 
             if ids is not None and ids.shape[0] >= minimum_points_per_frame:
 
@@ -432,23 +440,26 @@ def detect_points_in_stereo_canonical_space(left_point_detector,
     right_video_data.reinit()
 
     for j, _ in enumerate(left_images):
+
         left_undistorted = cv2.undistort(
             left_images[j],
             left_camera_matrix,
-            left_distortion_coeffs,
-            left_camera_matrix
+            left_distortion_coeffs
         )
+
         left_ids, left_obj_pts, left_img_pts = \
-            left_point_detector.get_points(left_undistorted)
+            left_point_detector.get_points(
+                left_undistorted, is_distorted=False)
 
         right_undistorted = cv2.undistort(
             right_images[j],
             right_camera_matrix,
-            right_distortion_coeffs,
-            right_camera_matrix
+            right_distortion_coeffs
         )
+
         right_ids, right_obj_pts, right_img_pts = \
-            right_point_detector.get_points(right_undistorted)
+            right_point_detector.get_points(
+                right_undistorted, is_distorted=False)
 
         if left_ids is not None \
                 and left_ids.shape[0] >= minimum_points_per_frame \
@@ -462,12 +473,13 @@ def detect_points_in_stereo_canonical_space(left_point_detector,
             left_homography, _ = \
                 cv2.findHomography(left_common_points[0:, 0:2],
                                    left_common_points[0:, 2:4])
+
             left_warped = cv2.warpPerspective(left_undistorted,
                                               left_homography,
                                               reference_image_size,)
 
             left_ids, left_obj_pts, left_img_pts = \
-                left_point_detector.get_points(left_warped)
+                left_point_detector.get_points(left_warped, is_distorted=False)
 
             right_common_points = match_points_by_id(right_ids,
                                                      right_img_pts,
@@ -476,12 +488,14 @@ def detect_points_in_stereo_canonical_space(left_point_detector,
             right_homography, _ = \
                 cv2.findHomography(right_common_points[0:, 0:2],
                                    right_common_points[0:, 2:4])
+
             right_warped = cv2.warpPerspective(right_undistorted,
                                                right_homography,
                                                reference_image_size,)
 
             right_ids, right_obj_pts, right_img_pts = \
-                right_point_detector.get_points(right_warped)
+                right_point_detector.get_points(
+                    right_warped, is_distorted=False)
 
             if left_ids is not None \
                     and left_ids.shape[0] >= minimum_points_per_frame \
@@ -507,3 +521,6 @@ def detect_points_in_stereo_canonical_space(left_point_detector,
                                                       right_homography,
                                                       right_camera_matrix,
                                                       right_distortion_coeffs)
+            else:
+                LOGGER.debug("Skipping frame " + str(j)
+                             + ", as not enough points.")
